@@ -1,6 +1,6 @@
 /*
  * Copyright (c) 2006-2007 Benedikt Meurer <benny@xfce.org>
- * Copyright (c) 2007      The Xfce Development Team
+ * Copyright (c) 2019      The Xfce Development Team
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -182,8 +182,8 @@ xfce_dialog_show_help (GtkWindow   *parent,
  * Asks the user to visit the online documentation. If confirmed, it will open
  * the webbrowser and redirect the user to the correct location.
  *
- * Appart from the @component, @page and @offset the following information
- * is also send to the server: user language and the xfce_version_string()
+ * Apart from the @component, @page and @offset the following information
+ * is also sent to the server: user language and the xfce_version_string()
  * or @version if set.
  *
  * See also: xfce_dialog_show_help().
@@ -267,15 +267,13 @@ xfce_dialog_show_help_with_version (GtkWindow   *parent,
     primary = g_strdup (_("Do you want to read the manual online?"));
 
   dialog = xfce_message_dialog_new (parent,
-                                    _("Online Documentation"),
+                                    NULL,
                                     "dialog-question",
                                     primary,
                                     _("You will be redirected to the documentation website "
                                       "where the help pages are maintained and translated."),
-                                    "gtk-cancel",
+                                    _("_Cancel"),
                                     GTK_RESPONSE_NO,
-                                    XFCE_BUTTON_TYPE_MIXED,
-                                    "help-browser",
                                     _("_Read Online"),
                                     GTK_RESPONSE_YES,
                                     NULL);
@@ -287,7 +285,7 @@ xfce_dialog_show_help_with_version (GtkWindow   *parent,
 
   button = gtk_check_button_new_with_mnemonic (_("_Always go directly to the online documentation"));
   gtk_box_pack_end (GTK_BOX (message_box), button, FALSE, TRUE, 0);
-  g_object_set (G_OBJECT (button), "halign", GTK_ALIGN_END, "margin-start", 8, "margin-end", 8, NULL);
+  g_object_set (G_OBJECT (button), "halign", GTK_ALIGN_END, "margin-start", 6, "margin-end", 6, NULL);
   gtk_widget_set_hexpand (button, TRUE);
   g_signal_connect (G_OBJECT (button), "toggled",
       G_CALLBACK (xfce_dialog_show_help_auto_toggled), NULL);
@@ -331,7 +329,7 @@ xfce_dialog_show_info (GtkWindow   *parent,
   primary_text = g_strdup_vprintf (primary_format, args);
   va_end (args);
 
-  xfce_message_dialog (parent, _("Information"),
+  xfce_message_dialog (parent, NULL,
                        "dialog-information",
                        primary_text, secondary_text,
                        XFCE_BUTTON_TYPE_MIXED, "window-close-symbolic", _("Close"),
@@ -366,7 +364,7 @@ xfce_dialog_show_warning (GtkWindow   *parent,
   primary_text = g_strdup_vprintf (primary_format, args);
   va_end (args);
 
-  xfce_message_dialog (parent, _("Warning"),
+  xfce_message_dialog (parent, NULL,
                        "dialog-warning",
                        primary_text, secondary_text,
                        XFCE_BUTTON_TYPE_MIXED, "window-close-symbolic", _("Close"),
@@ -402,7 +400,7 @@ xfce_dialog_show_error (GtkWindow    *parent,
   primary_text = g_strdup_vprintf (primary_format, args);
   va_end (args);
 
-  xfce_message_dialog (parent, _("Error"),
+  xfce_message_dialog (parent, NULL,
                        "dialog-error",
                        primary_text, error ? error->message : NULL,
                        XFCE_BUTTON_TYPE_MIXED, "window-close-symbolic", _("Close"),
@@ -422,7 +420,7 @@ xfce_dialog_show_error (GtkWindow    *parent,
  * @primary_format : (allow-none): the printf()-style format for the dialog question.
  * @...            : (allow-none): argument list for the @primary_format.
  *
- * Runs a questions dialog, that has a 'Cancel' and a 'Confirm' button. The 'Confirm'
+ * Runs a questions dialog that has a 'Cancel' and a 'Confirm' button. The 'Confirm'
  * button text can be set by @action if given.
  *
  * If @stock_id is equal to #GTK_STOCK_YES, the 'Cancel' button becomes a 'No' button.
@@ -461,7 +459,7 @@ xfce_dialog_confirm (GtkWindow   *parent,
       no_stock_id = _("Cancel");
     }
 
-  response_id = xfce_message_dialog (parent, _("Question"),
+  response_id = xfce_message_dialog (parent, NULL,
                                      "dialog-question",
                                      primary_text,
                                      secondary_text,
@@ -480,7 +478,7 @@ xfce_dialog_confirm (GtkWindow   *parent,
  * xfce_dialog_confirm_close_tabs:
  * @parent              : (allow-none): transient parent of the dialog, or %NULL.
  * @num_tabs            : the number of open tabs for display to user
- * @show_confirm_box    : whether to ask user if they want this confirmation in future
+ * @show_confirm_box    : whether to ask the user if this confirmation shall be shown in the future
  * @confirm_box_checked : (allow-none): state of confirmation checkbox
  *
  * Runs a dialog to ask the user whether they want to close the whole window,
@@ -490,11 +488,11 @@ xfce_dialog_confirm (GtkWindow   *parent,
  * are @num_tabs open tabs. If @num_tabs is negative, then the message to the
  * user will state simply that there are "multiple open tabs".
  *
- * If @show_confirm_box is %TRUE, then a checkbox is added to the dialog to allow
+ * If @show_confirm_box is %TRUE a checkbox is added to the dialog to allow
  * the user to set whether they wish to see this dialog in future. The initial
- * state of the checkbox is determined by the value stored at @confirm_box_checked,
+ * state of the checkbox is determined by the value stored at @confirm_box_checked
  * and the value at @confirm_box_checked after returning records the state of the
- * checkbox. If @show_confirm_box is %FALSE, @confirm_box_checked is ignored, and
+ * checkbox. If @show_confirm_box is %FALSE, @confirm_box_checked is ignored and
  * may be %NULL.
  *
  * Return value: #GTK_RESPONSE_CANCEL if cancelled, #GTK_RESPONSE_YES if the user
@@ -515,31 +513,33 @@ xfce_dialog_confirm_close_tabs (GtkWindow *parent,
   g_return_val_if_fail (parent == NULL || GTK_IS_WINDOW (parent), GTK_RESPONSE_NONE);
   g_return_val_if_fail (!show_confirm_box || confirm_box_checked != NULL, GTK_RESPONSE_NONE);
 
-  primary_text = _("Close all tabs?");
+  primary_text = _("Close window with multiple tabs?");
   if (num_tabs < 0)
-    secondary_text = g_strdup (_("This window has multiple tabs open. Closing this window\n"
+    secondary_text = g_strdup (_("This window has multiple tabs open. Closing this window "
                                 "will also close all its tabs."));
   else
-    secondary_text = g_strdup_printf (_("This window has %d tabs open. Closing this window\n"
+    secondary_text = g_strdup_printf (_("This window has %d tabs open. Closing this window "
                                "will also close all its tabs."), num_tabs);
 
   warning_icon = "dialog-warning";
 
-  dialog = xfce_message_dialog_new (parent, _("Warning"),
+  dialog = xfce_message_dialog_new (parent, NULL,
                                     warning_icon,
                                     primary_text,
                                     secondary_text,
-                                    XFCE_BUTTON_TYPE_MIXED, "gtk-cancel", _("_Cancel"), GTK_RESPONSE_CANCEL,
-                                    XFCE_BUTTON_TYPE_MIXED, "application-exit", _("Close _Window"), GTK_RESPONSE_YES,
-                                    XFCE_BUTTON_TYPE_MIXED, "window-close-symbolic", _("Close T_ab"), GTK_RESPONSE_CLOSE,
+                                    XFCE_BUTTON_TYPE_MIXED, NULL, _("_Cancel"), GTK_RESPONSE_CANCEL,
+                                    XFCE_BUTTON_TYPE_MIXED, NULL, _("Close T_ab"), GTK_RESPONSE_CLOSE,
+                                    XFCE_BUTTON_TYPE_MIXED, NULL, _("Close _Window"), GTK_RESPONSE_YES,
                                     NULL);
 
   if (show_confirm_box)
     {
       checkbutton = gtk_check_button_new_with_mnemonic (_("Do _not ask me again"));
-      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbutton), *confirm_box_checked);
       vbox = gtk_dialog_get_content_area (GTK_DIALOG (dialog));
-      gtk_box_pack_start (GTK_BOX (vbox), checkbutton, FALSE, FALSE, 5);
+      gtk_box_pack_end (GTK_BOX (vbox), checkbutton, FALSE, FALSE, 5);
+      g_object_set (G_OBJECT (checkbutton), "halign", GTK_ALIGN_END, "margin-start", 6, "margin-end", 6, NULL);
+      gtk_widget_set_hexpand (checkbutton, TRUE);
+      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbutton), *confirm_box_checked);
     }
 
   gtk_widget_show_all (dialog);
@@ -584,6 +584,7 @@ xfce_message_dialog_new_valist (GtkWindow   *parent,
   GtkWidget   *dialog_image;
   GtkWidget   *image;
   GtkWidget   *button;
+  GtkWidget   *content_area;
   GtkWidget   *label_box;
   const gchar *text = first_button_text;
   const gchar *label;
@@ -591,6 +592,7 @@ xfce_message_dialog_new_valist (GtkWindow   *parent,
   gint         response;
   GdkPixbuf   *pixbuf, *scaled;
   gint         w, h;
+  GList       *children;
 
   g_return_val_if_fail (primary_text != NULL || secondary_text != NULL, NULL);
   g_return_val_if_fail (parent == NULL || GTK_IS_WINDOW (parent), NULL);
@@ -603,6 +605,11 @@ xfce_message_dialog_new_valist (GtkWindow   *parent,
   dialog_image = GTK_WIDGET(gtk_builder_get_object(gxml, "icon_stock_id"));
   gtk_window_set_default_size (GTK_WINDOW (dialog), 500, -1);
 
+  /* Remove the original message area of the GtkMessageDialog as we add our own */
+  content_area = GTK_WIDGET (gtk_dialog_get_content_area (GTK_DIALOG (dialog)));
+  children = gtk_container_get_children (GTK_CONTAINER (content_area));
+  gtk_container_remove (GTK_CONTAINER (content_area), GTK_WIDGET ((g_list_nth (children, 1))->data));
+
   if (parent)
     {
       gtk_window_set_transient_for (GTK_WINDOW (dialog), parent);
@@ -635,6 +642,7 @@ xfce_message_dialog_new_valist (GtkWindow   *parent,
       GtkWidget *secondary_label = gtk_label_new (secondary_text);
       gtk_label_set_line_wrap (GTK_LABEL (secondary_label), TRUE);
       gtk_label_set_xalign (GTK_LABEL (secondary_label), 0.0);
+      gtk_label_set_max_width_chars (GTK_LABEL (secondary_label), 80);
       gtk_widget_set_vexpand (GTK_WIDGET (secondary_label), TRUE);
       gtk_widget_set_valign (GTK_WIDGET (secondary_label), 0.0);
 
@@ -735,7 +743,7 @@ xfce_message_dialog_new_valist (GtkWindow   *parent,
  * @...               : %NULL terminated list of parameters.
  *
  * xfce_message_dialog_new() allows you to easily create Gtk+ message dialogs.
- * It accepts GTK+ stock buttons, mixed buttons (using XFCE_BUTTON_TYPE_MIXED)
+ * It accepts GTK+ stock buttons (deprecated), mixed buttons (using XFCE_BUTTON_TYPE_MIXED)
  * or buttons with a #GdkPixbuf (using XFCE_BUTTON_TYPE_PIXBUF).
  *
  * The buttons are defined by @first_button_text and the next arguments in the
@@ -747,7 +755,7 @@ xfce_message_dialog_new_valist (GtkWindow   *parent,
  *     <listitem>
  *       <para>
  *         This allows you to easily create mixed buttons in a dialog.
- *         @param1 is used for the stock_id, @param2 for the label and
+ *         @param1 is used for the icon name, @param2 for the label and
  *         @param3 for the response_id. See also xfce_gtk_button_new_mixed().
  *       </para>
  *     </listitem>
@@ -787,7 +795,7 @@ xfce_message_dialog_new_valist (GtkWindow   *parent,
  *                                          "There are unsaved modifications",
  *                                          "The menu has been modified, do you want to save it before quitting?",
  *                                          GTK_STOCK_SAVE, GTK_RESPONSE_YES,
- *                                          XFCE_BUTTON_TYPE_MIXED, GTK_STOCK_DELETE, "Forget modifications", GTK_RESPONSE_APPLY,
+ *                                          XFCE_BUTTON_TYPE_MIXED, "edit-delete", _("Forget modifications"), GTK_RESPONSE_APPLY,
  *                                          XFCE_BUTTON_TYPE_PIXBUF, pixbuf, "Quit", GTK_RESPONSE_NO,
  *                                          GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
  *                                          NULL);
